package com.battlelancer.seriesguide.ui.movies

import android.content.Intent
import android.graphics.Bitmap
import android.graphics.Color
import android.graphics.drawable.BitmapDrawable
import android.graphics.drawable.ColorDrawable
import android.os.AsyncTask
import android.os.Build
import android.os.Bundle
import android.os.Handler
import android.view.LayoutInflater
import android.view.Menu
import android.view.MenuInflater
import android.view.MenuItem
import android.view.View
import android.view.ViewGroup
import androidx.appcompat.app.AppCompatActivity
import androidx.collection.SparseArrayCompat
import androidx.core.content.ContextCompat
import androidx.core.graphics.ColorUtils
import androidx.core.view.isGone
import androidx.core.widget.NestedScrollView
import androidx.fragment.app.Fragment
import androidx.loader.app.LoaderManager
import androidx.loader.content.Loader
import androidx.palette.graphics.Palette
import com.battlelancer.seriesguide.R
import com.battlelancer.seriesguide.backend.settings.HexagonSettings
import com.battlelancer.seriesguide.databinding.FragmentMovieBinding
import com.battlelancer.seriesguide.extensions.ActionsHelper
import com.battlelancer.seriesguide.extensions.ExtensionManager
import com.battlelancer.seriesguide.extensions.MovieActionsContract
import com.battlelancer.seriesguide.settings.TmdbSettings
import com.battlelancer.seriesguide.streaming.StreamingSearch
import com.battlelancer.seriesguide.streaming.StreamingSearchConfigureDialog
import com.battlelancer.seriesguide.traktapi.MovieCheckInDialogFragment
import com.battlelancer.seriesguide.traktapi.RateDialogFragment
import com.battlelancer.seriesguide.traktapi.TraktCredentials
import com.battlelancer.seriesguide.traktapi.TraktTools
import com.battlelancer.seriesguide.ui.BaseNavDrawerActivity
import com.battlelancer.seriesguide.ui.FullscreenImageActivity
import com.battlelancer.seriesguide.ui.comments.TraktCommentsActivity
import com.battlelancer.seriesguide.ui.people.MovieCreditsLoader
import com.battlelancer.seriesguide.ui.people.PeopleListHelper
import com.battlelancer.seriesguide.util.LanguageTools
import com.battlelancer.seriesguide.util.ServiceUtils
import com.battlelancer.seriesguide.util.ShareUtils
import com.battlelancer.seriesguide.util.TextTools
import com.battlelancer.seriesguide.util.TimeTools
import com.battlelancer.seriesguide.util.TmdbTools
import com.battlelancer.seriesguide.util.Utils
import com.battlelancer.seriesguide.util.ViewTools
import com.battlelancer.seriesguide.util.copyTextToClipboardOnLongClick
import com.squareup.picasso.Callback
import com.squareup.picasso.Picasso
import com.uwetrottmann.androidutils.AndroidUtils
import com.uwetrottmann.androidutils.CheatSheet
import com.uwetrottmann.tmdb2.entities.Credits
import com.uwetrottmann.tmdb2.entities.Videos
import org.greenrobot.eventbus.EventBus
import org.greenrobot.eventbus.Subscribe
import org.greenrobot.eventbus.ThreadMode
import timber.log.Timber
import java.util.ArrayList

/**
 * Displays details about one movie including plot, ratings, trailers and a poster.
 */
class MovieDetailsFragment : Fragment(), MovieActionsContract {

    private var _binding: FragmentMovieBinding? = null
    private val binding get() = _binding!!

    private var tmdbId: Int = 0
    private var movieDetails: MovieDetails? = MovieDetails()
    private var movieTitle: String? = null
    private var trailer: Videos.Video? = null

    private val handler = Handler()
    private var paletteAsyncTask: AsyncTask<Bitmap, Void, Palette>? = null

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        _binding = FragmentMovieBinding.inflate(inflater, container, false)
        val view = binding.root

        binding.progressBar.isGone = true
        binding.textViewMovieGenresLabel.isGone = true

        // important action buttons
        binding.containerMovieButtons.root.isGone = true
        binding.containerMovieButtons.buttonMovieCheckIn.setOnClickListener { onButtonCheckInClick() }
        binding.containerMovieButtons.buttonMovieStreamingSearch.apply {
            setOnClickListener { onButtonStreamingSearchClick() }
            setOnLongClickListener { onButtonStreamingSearchLongClick() }
        }
        binding.containerRatings.root.isGone = true
        CheatSheet.setup(binding.containerMovieButtons.buttonMovieCheckIn)

        // language button
        binding.buttonMovieLanguage.isGone = true
        CheatSheet.setup(binding.buttonMovieLanguage, R.string.pref_language)
        binding.buttonMovieLanguage.setOnClickListener {
            MovieLocalizationDialogFragment.show(fragmentManager)
        }

        // comments button
        binding.buttonMovieComments.isGone = true

        // cast and crew
        setCastVisibility(false)
        setCrewVisibility(false)

        // set up long-press to copy text to clipboard (d-pad friendly vs text selection)
        binding.textViewMovieTitle.copyTextToClipboardOnLongClick()
        binding.textViewMovieDate.copyTextToClipboardOnLongClick()
        binding.textViewMovieDescription.copyTextToClipboardOnLongClick()
        binding.textViewMovieGenres.copyTextToClipboardOnLongClick()

        return view
    }

    override fun onActivityCreated(savedInstanceState: Bundle?) {
        super.onActivityCreated(savedInstanceState)

        tmdbId = arguments!!.getInt(ARG_TMDB_ID)
        if (tmdbId <= 0) {
            fragmentManager!!.popBackStack()
            return
        }

        setupViews()

        val args = Bundle()
        args.putInt(ARG_TMDB_ID, tmdbId)
        LoaderManager.getInstance(this).apply {
            initLoader(MovieDetailsActivity.LOADER_ID_MOVIE, args, movieLoaderCallbacks)
            initLoader(
                MovieDetailsActivity.LOADER_ID_MOVIE_TRAILERS, args, trailerLoaderCallbacks
            )
            initLoader(MovieDetailsActivity.LOADER_ID_MOVIE_CREDITS, args, creditsLoaderCallbacks)
        }

        setHasOptionsMenu(true)
    }

    private fun setupViews() {
        // avoid overlap with status + action bar (adjust top margin)
        // warning: pre-M status bar not always translucent (e.g. Nexus 10)
        // (using fitsSystemWindows would not work correctly with multiple views)
        val config = (activity as MovieDetailsActivity).systemBarTintManager.config
        val pixelInsetTop = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            config.statusBarHeight // full screen, status bar transparent
        } else {
            config.getPixelInsetTop(false) // status bar translucent
        }

        // action bar height is pre-set as top margin, add to it
        val decorationHeightPx = pixelInsetTop + binding.contentContainerMovie.paddingTop
        binding.contentContainerMovie.setPadding(0, decorationHeightPx, 0, 0)

        // dual pane layout?
        binding.contentContainerMovieRight?.setPadding(0, decorationHeightPx, 0, 0)

        // show toolbar title and background when scrolling
        val defaultPaddingPx = resources.getDimensionPixelSize(R.dimen.default_padding)
        val scrollChangeListener = ToolbarScrollChangeListener(defaultPaddingPx, decorationHeightPx)
        binding.contentContainerMovie.setOnScrollChangeListener(scrollChangeListener)
        binding.contentContainerMovieRight?.setOnScrollChangeListener(scrollChangeListener)
    }

    override fun onStart() {
        super.onStart()

        val event = EventBus.getDefault()
            .getStickyEvent(BaseNavDrawerActivity.ServiceActiveEvent::class.java)
        setMovieButtonsEnabled(event == null)

        EventBus.getDefault().register(this)
    }

    override fun onResume() {
        super.onResume()

        // refresh actions when returning, enabled extensions or their actions might have changed
        loadMovieActionsDelayed()
    }

    override fun onStop() {
        super.onStop()

        EventBus.getDefault().unregister(this)
    }

    override fun onDestroyView() {
        super.onDestroyView()

        // Always cancel the request here, this is safe to call even if the image has been loaded.
        // This ensures that the anonymous callback we have does not prevent the fragment from
        // being garbage collected. It also prevents our callback from getting invoked even after the
        // fragment is destroyed.
        Picasso.get().cancelRequest(binding.imageViewMoviePoster)
        // same for Palette task
        paletteAsyncTask?.cancel(true)

        _binding = null
    }

    override fun onCreateOptionsMenu(menu: Menu, inflater: MenuInflater) {
        super.onCreateOptionsMenu(menu, inflater)

        movieDetails?.let {
            inflater.inflate(R.menu.movie_details_menu, menu)

            // enable/disable actions
            val isEnableShare = !it.tmdbMovie()?.title.isNullOrEmpty()
            menu.findItem(R.id.menu_movie_share).apply {
                isEnabled = isEnableShare
                isVisible = isEnableShare
            }

            val isEnableImdb = !it.tmdbMovie()?.imdb_id.isNullOrEmpty()
            menu.findItem(R.id.menu_open_imdb).apply {
                isEnabled = isEnableImdb
                isVisible = isEnableImdb
            }

            val isEnableYoutube = trailer != null
            menu.findItem(R.id.menu_open_youtube).apply {
                isEnabled = isEnableYoutube
                isVisible = isEnableYoutube
            }
        }
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        val itemId = item.itemId
        if (itemId == R.id.menu_movie_share) {
            movieDetails?.tmdbMovie()?.let { ShareUtils.shareMovie(activity, tmdbId, it.title) }
            return true
        }
        if (itemId == R.id.menu_open_imdb) {
            movieDetails?.tmdbMovie()?.let { ServiceUtils.openImdb(it.imdb_id, activity) }
            return true
        }
        if (itemId == R.id.menu_open_youtube) {
            trailer?.let { ServiceUtils.openYoutube(it.key, activity) }
            return true
        }
        if (itemId == R.id.menu_open_tmdb) {
            TmdbTools.openTmdbMovie(activity, tmdbId)
        }
        if (itemId == R.id.menu_open_trakt) {
            Utils.launchWebsite(activity, TraktTools.buildMovieUrl(tmdbId))
            return true
        }
        return super.onOptionsItemSelected(item)
    }

    private fun populateMovieViews() {
        val movieDetails = this.movieDetails ?: return
        /*
          Get everything from TMDb. Also get additional rating from trakt.
         */
        val tmdbMovie = movieDetails.tmdbMovie() ?: return
        val traktRatings = movieDetails.traktRatings()
        val inCollection = movieDetails.isInCollection
        val inWatchlist = movieDetails.isInWatchlist
        val isWatched = movieDetails.isWatched
        val rating = movieDetails.userRating

        movieTitle = tmdbMovie.title
        binding.textViewMovieTitle.text = tmdbMovie.title
        activity!!.title = tmdbMovie.title
        binding.textViewMovieDescription.text = TextTools.textWithTmdbSource(
            binding.textViewMovieDescription.context,
            tmdbMovie.overview
        )

        // release date and runtime: "July 17, 2009 | 95 min"
        val releaseAndRuntime = StringBuilder()
        tmdbMovie.release_date?.let {
            releaseAndRuntime.append(TimeTools.formatToLocalDate(context, it))
            releaseAndRuntime.append(" | ")
        }
        tmdbMovie.runtime?.let {
            releaseAndRuntime.append(getString(R.string.runtime_minutes, it.toString()))
        }
        binding.textViewMovieDate.text = releaseAndRuntime.toString()

        // hide check-in if not connected to trakt or hexagon is enabled
        val isConnectedToTrakt = TraktCredentials.get(activity).hasCredentials()
        val hideCheckIn = !isConnectedToTrakt || HexagonSettings.isEnabled(activity)
        binding.containerMovieButtons.buttonMovieCheckIn.isGone = hideCheckIn
        // hide streaming search if turned off
        val hideStreamingSearch = StreamingSearch.isTurnedOff(requireContext())
        binding.containerMovieButtons.buttonMovieStreamingSearch.isGone = hideStreamingSearch
        binding.containerMovieButtons.dividerMovieButtons.isGone = hideCheckIn && hideStreamingSearch

        // watched button
        binding.containerMovieButtons.buttonMovieWatched.also {
            val textRes = if (isWatched) R.string.action_unwatched else R.string.action_watched
            it.setText(textRes)
            CheatSheet.setup(it, textRes)
            if (isWatched) {
                ViewTools.setVectorDrawableTop(it, R.drawable.ic_watched_24dp)
            } else {
                ViewTools.setVectorDrawableTop(it, R.drawable.ic_watch_black_24dp)
            }
            it.setOnClickListener {
                if (isWatched) {
                    MovieTools.unwatchedMovie(context, tmdbId)
                } else {
                    MovieTools.watchedMovie(context, tmdbId, inWatchlist)
                }
            }
        }

        // collected button
        binding.containerMovieButtons.buttonMovieCollected.also {
            if (inCollection) {
                ViewTools.setVectorDrawableTop(it, R.drawable.ic_collected_24dp)
            } else {
                ViewTools.setVectorDrawableTop(it, R.drawable.ic_collect_black_24dp)
            }
            val textRes =
                if (inCollection) R.string.action_collection_remove else R.string.action_collection_add
            it.setText(textRes)
            CheatSheet.setup(it, textRes)
            it.setOnClickListener {
                if (inCollection) {
                    MovieTools.removeFromCollection(context, tmdbId)
                } else {
                    MovieTools.addToCollection(context, tmdbId)
                }
            }
        }

        // watchlist button
        binding.containerMovieButtons.buttonMovieWatchlisted.also {
            if (inWatchlist) {
                ViewTools.setVectorDrawableTop(it, R.drawable.ic_list_added_24dp)
            } else {
                ViewTools.setVectorDrawableTop(it, R.drawable.ic_list_add_white_24dp)
            }
            val textRes = if (inWatchlist) R.string.watchlist_remove else R.string.watchlist_add
            it.setText(textRes)
            CheatSheet.setup(it, textRes)
            it.setOnClickListener {
                if (inWatchlist) {
                    MovieTools.removeFromWatchlist(context, tmdbId)
                } else {
                    MovieTools.addToWatchlist(context, tmdbId)
                }
            }
        }

        // show button bar
        binding.containerMovieButtons.root.isGone = false

        // language button
        binding.buttonMovieLanguage.also {
            val languageData = LanguageTools.getMovieLanguageData(context)
            it.text = languageData?.languageString
            it.isGone = false
        }

        // ratings
        binding.containerRatings.textViewRatingsTmdbValue.text = TraktTools.buildRatingString(tmdbMovie.vote_average)
        binding.containerRatings.textViewRatingsTmdbVotes.text =
            TraktTools.buildRatingVotesString(activity, tmdbMovie.vote_count)
        traktRatings?.let {
            binding.containerRatings.textViewRatingsTraktVotes.text =
                TraktTools.buildRatingVotesString(activity, it.votes)
            binding.containerRatings.textViewRatingsTraktValue.text = TraktTools.buildRatingString(it.rating)
        }
        // if movie is not in database, can't handle user ratings
        if (!inCollection && !inWatchlist && !isWatched) {
            binding.containerRatings.textViewRatingsTraktUserLabel.isGone = true
            binding.containerRatings.textViewRatingsTraktUser.isGone = true
            binding.containerRatings.root.isClickable = false
            binding.containerRatings.root.isLongClickable = false // cheat sheet
        } else {
            binding.containerRatings.textViewRatingsTraktUserLabel.isGone = false
            binding.containerRatings.textViewRatingsTraktUser.isGone = false
            binding.containerRatings.textViewRatingsTraktUser.text = TraktTools.buildUserRatingString(activity, rating)
            binding.containerRatings.root.setOnClickListener { rateMovie() }
            CheatSheet.setup(binding.containerRatings.root, R.string.action_rate)
        }
        binding.containerRatings.root.isGone = false

        // genres
        binding.textViewMovieGenresLabel.isGone = false
        ViewTools.setValueOrPlaceholder(
            binding.textViewMovieGenres, TmdbTools.buildGenresString(tmdbMovie.genres)
        )

        // trakt comments link
        binding.buttonMovieComments.setOnClickListener { v ->
            val i = Intent(activity, TraktCommentsActivity::class.java)
            i.putExtras(TraktCommentsActivity.createInitBundleMovie(movieTitle, tmdbId))
            Utils.startActivityWithAnimation(activity, i, v)
        }
        binding.buttonMovieComments.isGone = false

        // load poster, cache on external storage
        if (tmdbMovie.poster_path.isNullOrEmpty()) {
            binding.frameLayoutMoviePoster.isClickable = false
            binding.frameLayoutMoviePoster.isFocusable = false
        } else {
            val smallImageUrl = (TmdbSettings.getImageBaseUrl(activity)
                    + TmdbSettings.POSTER_SIZE_SPEC_W342 + tmdbMovie.poster_path)
            ServiceUtils.loadWithPicasso(activity, smallImageUrl)
                .into(binding.imageViewMoviePoster, object : Callback.EmptyCallback() {
                    override fun onSuccess() {
                        val bitmap = (binding.imageViewMoviePoster.drawable as BitmapDrawable).bitmap
                        paletteAsyncTask = Palette.from(bitmap)
                            .generate { palette ->
                                if (palette != null) {
                                    var color = palette.getVibrantColor(Color.WHITE)
                                    color = ColorUtils.setAlphaComponent(color, 50)
                                    binding.rootLayoutMovie.setBackgroundColor(color)
                                }
                            }
                    }
                })
            // click listener for high resolution poster
            binding.frameLayoutMoviePoster.also {
                it.isFocusable = true
                it.setOnClickListener { view ->
                    val largeImageUrl = (TmdbSettings.getImageBaseUrl(activity)
                            + TmdbSettings.POSTER_SIZE_SPEC_ORIGINAL + tmdbMovie.poster_path)
                    val intent = Intent(activity, FullscreenImageActivity::class.java).apply {
                        putExtra(FullscreenImageActivity.EXTRA_PREVIEW_IMAGE, smallImageUrl)
                        putExtra(FullscreenImageActivity.EXTRA_IMAGE, largeImageUrl)
                    }
                    Utils.startActivityWithAnimation(activity, intent, view)
                }
            }
        }
    }

    private fun populateMovieCreditsViews(credits: Credits?) {
        if (credits == null) {
            setCastVisibility(false)
            setCrewVisibility(false)
            return
        }

        // cast members
        if (credits.cast?.size != 0
            && PeopleListHelper.populateMovieCast(activity, binding.moviePeople.containerCast, credits)) {
            setCastVisibility(true)
        } else {
            setCastVisibility(false)
        }

        // crew members
        if (credits.crew?.size != 0
            && PeopleListHelper.populateMovieCrew(activity, binding.moviePeople.containerCrew, credits)) {
            setCrewVisibility(true)
        } else {
            setCrewVisibility(false)
        }
    }

    private fun onButtonCheckInClick() {
        movieTitle?.let {
            if (it.isEmpty()) {
                return
            }
            MovieCheckInDialogFragment.show(fragmentManager, tmdbId, it)
        }
    }

    private fun onButtonStreamingSearchClick() {
        movieTitle?.let {
            if (it.isEmpty()) {
                return
            }
            if (StreamingSearch.isNotConfigured(requireContext())) {
                showStreamingSearchConfigDialog()
            } else {
                StreamingSearch.searchForMovie(requireContext(), it)
            }
        }
    }

    private fun onButtonStreamingSearchLongClick(): Boolean {
        showStreamingSearchConfigDialog()
        return true
    }

    private fun showStreamingSearchConfigDialog() {
        StreamingSearchConfigureDialog.show(requireFragmentManager())
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    fun onStreamingSearchConfigured(
        event: StreamingSearchConfigureDialog.StreamingSearchConfiguredEvent
    ) {
        if (event.turnedOff) {
            binding.containerMovieButtons.buttonMovieStreamingSearch.isGone = true
        } else {
            onButtonStreamingSearchClick()
        }
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    override fun onEventMainThread(event: ExtensionManager.MovieActionReceivedEvent) {
        if (event.movieTmdbId != tmdbId) {
            return
        }
        loadMovieActionsDelayed()
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    fun onEvent(event: MovieTools.MovieChangedEvent) {
        if (event.movieTmdbId != tmdbId) {
            return
        }
        // re-query to update movie details
        restartMovieLoader()
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    fun onEventEpisodeTask(@Suppress("UNUSED_PARAMETER") event: BaseNavDrawerActivity.ServiceActiveEvent) {
        setMovieButtonsEnabled(false)
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    fun onEventEpisodeTask(@Suppress("UNUSED_PARAMETER") event: BaseNavDrawerActivity.ServiceCompletedEvent) {
        setMovieButtonsEnabled(true)
    }

    private fun setMovieButtonsEnabled(enabled: Boolean) {
        binding.containerMovieButtons.apply {
            buttonMovieCheckIn.isEnabled = enabled
            buttonMovieWatched.isEnabled = enabled
            buttonMovieCollected.isEnabled = enabled
            buttonMovieWatchlisted.isEnabled = enabled
            buttonMovieStreamingSearch.isEnabled = enabled
        }
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    fun handleLanguageEvent(@Suppress("UNUSED_PARAMETER") event: MovieLocalizationDialogFragment.LocalizationChangedEvent) {
        // reload movie details and trailers (but not cast/crew info which is not language dependent)
        restartMovieLoader()
        val args = Bundle().apply {
            putInt(ARG_TMDB_ID, tmdbId)
        }
        LoaderManager.getInstance(this).restartLoader(
            MovieDetailsActivity.LOADER_ID_MOVIE_TRAILERS, args, trailerLoaderCallbacks
        )
    }

    override fun loadMovieActions() {
        var actions = ExtensionManager.get(context)
            .getLatestMovieActions(context, tmdbId)

        // no actions available yet, request extensions to publish them
        if (actions == null || actions.size == 0) {
            actions = ArrayList()

            movieDetails?.tmdbMovie()?.let {
                val movie = com.battlelancer.seriesguide.api.Movie.Builder()
                    .tmdbId(tmdbId)
                    .imdbId(it.imdb_id)
                    .title(it.title)
                    .releaseDate(it.release_date)
                    .build()
                ExtensionManager.get(context).requestMovieActions(context, movie)
            }
        }

        Timber.d("loadMovieActions: received %s actions for %s", actions.size, tmdbId)
        ActionsHelper.populateActions(
            activity!!.layoutInflater, activity!!.theme, binding.containerMovieActions, actions
        )
    }

    private val movieActionsRunnable = Runnable {
        if (!isAdded) {
            return@Runnable // we need an activity for this, abort.
        }
        loadMovieActions()
    }

    override fun loadMovieActionsDelayed() {
        handler.removeCallbacks(movieActionsRunnable)
        handler.postDelayed(
            movieActionsRunnable, MovieActionsContract.ACTION_LOADER_DELAY_MILLIS.toLong()
        )
    }

    private fun rateMovie() {
        RateDialogFragment.newInstanceMovie(tmdbId).safeShow(context, fragmentManager)
    }

    private fun setCrewVisibility(visible: Boolean) {
        binding.moviePeople.labelCrew.isGone = !visible
        binding.moviePeople.containerCrew.isGone = !visible
    }

    private fun setCastVisibility(visible: Boolean) {
        binding.moviePeople.labelCast.isGone = !visible
        binding.moviePeople.containerCast.isGone = !visible
    }

    private fun restartMovieLoader() {
        val args = Bundle()
        args.putInt(ARG_TMDB_ID, tmdbId)
        LoaderManager.getInstance(this)
            .restartLoader(MovieDetailsActivity.LOADER_ID_MOVIE, args, movieLoaderCallbacks)
    }

    private val movieLoaderCallbacks = object : LoaderManager.LoaderCallbacks<MovieDetails> {
        override fun onCreateLoader(loaderId: Int, args: Bundle?): Loader<MovieDetails> {
            binding.progressBar.isGone = false
            return MovieLoader(context, args!!.getInt(ARG_TMDB_ID))
        }

        override fun onLoadFinished(movieLoader: Loader<MovieDetails>, movieDetails: MovieDetails) {
            if (!isAdded) {
                return
            }
            this@MovieDetailsFragment.movieDetails = movieDetails
            binding.progressBar.isGone = true

            // we need at least values from database or tmdb
            if (movieDetails.tmdbMovie() != null) {
                populateMovieViews()
                loadMovieActions()
                activity!!.invalidateOptionsMenu()
            } else {
                // if there is no local data and loading from network failed
                binding.textViewMovieDescription.text = if (AndroidUtils.isNetworkConnected(context)) {
                    getString(R.string.api_error_generic, getString(R.string.tmdb))
                } else {
                    getString(R.string.offline)
                }
            }
        }

        override fun onLoaderReset(movieLoader: Loader<MovieDetails>) {
            // nothing to do
        }
    }

    private val trailerLoaderCallbacks = object : LoaderManager.LoaderCallbacks<Videos.Video> {
        override fun onCreateLoader(loaderId: Int, args: Bundle?): Loader<Videos.Video> {
            return MovieTrailersLoader(context, args!!.getInt(ARG_TMDB_ID))
        }

        override fun onLoadFinished(
            trailersLoader: Loader<Videos.Video>,
            trailer: Videos.Video?
        ) {
            if (!isAdded) {
                return
            }
            if (trailer != null) {
                this@MovieDetailsFragment.trailer = trailer
                activity!!.invalidateOptionsMenu()
            }
        }

        override fun onLoaderReset(trailersLoader: Loader<Videos.Video>) {
            // do nothing
        }
    }

    private val creditsLoaderCallbacks = object : LoaderManager.LoaderCallbacks<Credits?> {
        override fun onCreateLoader(loaderId: Int, args: Bundle?): Loader<Credits?> {
            return MovieCreditsLoader(context!!, args!!.getInt(ARG_TMDB_ID))
        }

        override fun onLoadFinished(creditsLoader: Loader<Credits?>, credits: Credits?) {
            if (!isAdded) {
                return
            }
            populateMovieCreditsViews(credits)
        }

        override fun onLoaderReset(creditsLoader: Loader<Credits?>) {
            // do nothing
        }
    }

    private inner class ToolbarScrollChangeListener(
        private val overlayThresholdPx: Int,
        private val titleThresholdPx: Int
    ) : NestedScrollView.OnScrollChangeListener {

        // we have determined by science that a capacity of 2 is good in our case :)
        private val showOverlayMap: SparseArrayCompat<Boolean> = SparseArrayCompat(2)
        private var showOverlay: Boolean = false
        private var showTitle: Boolean = false

        override fun onScrollChange(
            v: NestedScrollView, scrollX: Int, scrollY: Int, oldScrollX: Int, oldScrollY: Int
        ) {
            val actionBar = (activity as AppCompatActivity?)?.supportActionBar ?: return

            val viewId = v.id

            var shouldShowOverlayTemp = scrollY > overlayThresholdPx
            showOverlayMap.put(viewId, shouldShowOverlayTemp)
            for (i in 0 until showOverlayMap.size()) {
                shouldShowOverlayTemp = shouldShowOverlayTemp or showOverlayMap.valueAt(i)
            }
            val shouldShowOverlay = shouldShowOverlayTemp

            if (!showOverlay && shouldShowOverlay) {
                val primaryColor = ContextCompat.getColor(
                    v.context,
                    Utils.resolveAttributeToResourceId(
                        v.context.theme, R.attr.sgColorStatusBarOverlay
                    )
                )
                actionBar.setBackgroundDrawable(ColorDrawable(primaryColor))
            } else if (showOverlay && !shouldShowOverlay) {
                actionBar.setBackgroundDrawable(null)
            }
            showOverlay = shouldShowOverlay

            // only main container should show/hide title
            if (viewId == R.id.contentContainerMovie) {
                val shouldShowTitle = scrollY > titleThresholdPx
                if (!showTitle && shouldShowTitle) {
                    movieDetails?.tmdbMovie()?.let {
                        actionBar.title = it.title
                        actionBar.setDisplayShowTitleEnabled(true)
                    }
                } else if (showTitle && !shouldShowTitle) {
                    actionBar.setDisplayShowTitleEnabled(false)
                }
                showTitle = shouldShowTitle
            }
        }
    }

    companion object {

        const val ARG_TMDB_ID = "tmdbid"

        @JvmStatic
        fun newInstance(tmdbId: Int): MovieDetailsFragment {
            return MovieDetailsFragment().apply {
                arguments = Bundle().apply {
                    putInt(ARG_TMDB_ID, tmdbId)
                }
            }
        }
    }
}
